"""Template tag library around URL transformations.

Provides one template tag: "update_query_string".

Example:

.. code-block:: Django

   {% load updateurl %}

   First example using strings:
   {% update_query_string with 'page'='2' %}

   Second example using variables (both sides of "=" can be variables):
   {% update_query_string with page=paginator.next_page %}

"""

import re
import urllib

from django import template
from django.template.base import FilterExpression
from django.utils.html import conditional_escape


register = template.Library()


def token_value(bits, parser):
    """Parse ``bits`` string and return string or variable (FilterExpression).

    """
    if bits[0] in ('"', "'"):  # Parse a string.
        if not (bits[0] == bits[-1] or len(bits) < 2):
            raise template.TemplateSyntaxError("Malformed argument %r" % bits)
        return bits[1:-1]
    else:  # Parse a variable.
        return FilterExpression(bits, parser)


def token_kwargs(bits, parser):
    """Return dictionary of keywords arguments from ``bits``.

    bits
      A list containing remainder of the token (split by spaces) that is to be
      checked for arguments. Valid arguments will be removed from this list.

    parser
      A template parser instance.

    There is no requirement for all remaining token ``bits`` to be keyword
    arguments, so the dictionary will be returned as soon as an invalid
    argument format is reached.

    .. note::

       This function is a fork of :py:func:`django.template.base.token_kwargs`
       that adds support for variables both sides of the "=" assignation
       operator.

    """
    if not bits:
        return {}
    kwarg_re = re.compile(r"(?P<key>.+)=(?P<value>.+)")
    kwargs = {}
    while bits:
        match = kwarg_re.match(bits[0])
        if not match or not match.group('key') or not match.group('value'):
            return kwargs
        key = token_value(match.group('key'), parser)
        value = token_value(match.group('value'), parser)
        kwargs[key] = value
        del bits[:1]
    return kwargs


def update_query_string(url, updates):
    """Update query string in ``url`` with ``updates``.

    """
    url_parts = list(urllib.parse.urlparse(str(url)))
    query_dict = urllib.parse.parse_qs(url_parts[4])
    query_dict.update(updates)
    query_string = urllib.parse.urlencode(query_dict, True)
    url_parts[4] = query_string
    return urllib.parse.urlunparse(url_parts)


class UpdateQueryStringNode(template.Node):
    def __init__(self, url=None, qs_updates={}):
        self.url = url
        self.qs_updates = qs_updates

    def render(self, context):
        if self.url is None:  # Fallback to current URL.
            request = context['request']
            url = request.get_full_path()
        else:
            url = self.url
        try:
            url = url.resolve(context)
        except AttributeError:
            url = str(url)
        updates = {}
        for key, value in self.qs_updates.items():
            try:
                key = key.resolve(context)
            except AttributeError:
                key = str(key)
            try:
                value = value.resolve(context)
            except AttributeError:
                value = str(value)
            updates[key] = value
        new_url = update_query_string(url, updates)
        try:
            do_escape = context.autoescape
        except AttributeError:
            do_escape = True  # By default, escape.
        if do_escape:
            new_url = conditional_escape(new_url)
        return new_url


@register.tag('update_query_string')
def tag_update_query_string(parser, token):
    """Return URL with updated querystring.

    """
    bits = token.split_contents()
    # Quickly check number of arguments.
    if len(bits) < 2:
        raise template.TemplateSyntaxError("'update_query_string' tag "
                                           "requires at least two arguments")
    # Parse options.
    options = {}
    tag_name = bits[0]
    remaining_bits = bits[1:]
    while remaining_bits:
        option = remaining_bits.pop(0)
        if option in options:
            raise template.TemplateSyntaxError(
                'The %r option was specified more than once.' % option)
        if option == 'with':  # "with" is followed by keyword arguments
            value = token_kwargs(remaining_bits, parser)
            if not value:
                raise template.TemplateSyntaxError('"with" in %r tag needs at '
                                                   'least one keyword '
                                                   'argument.' % tag_name)
        else:
            raise template.TemplateSyntaxError(
                'Unknown argument for %r tag: %r.' % (bits[0], option))
        options[option] = value
    qs_updates = options.get('with', {})
    url = None  # For now, we do not support URL as argument.
    return UpdateQueryStringNode(url=url, qs_updates=qs_updates)



import unittest

from django import forms
from django.template import Template, Context
from django.template.base import Parser, Variable
from django.test import RequestFactory

from django_genericfilters.templatetags.updateurl import (
    token_kwargs, token_value, update_query_string
)
from django_genericfilters.templatetags.utils import is_checkbox


class TemplateTagTestCase(unittest.TestCase):
    def test_token_value(self):
        parser = Parser('')
        self.assertEqual(token_value('"A"', parser), 'A')
        var = token_value('a', parser).var
        self.assertTrue(isinstance(var, Variable))

    def test_token_kwargs(self):
        parser = Parser('')
        self.assertEqual(token_kwargs([], parser), {})
        bits = ['a="A"']
        token_kwargs(bits, parser)
        self.assertEqual(bits, [])
        bits = ['a="A"', 'invalid']
        token_kwargs(bits, parser)
        self.assertEqual(bits, ['invalid'])

    def test_update_query_string(self):
        self.assertEqual(
            update_query_string('/foo/?bar=baz', {'bar': 'updated'}),
            '/foo/?bar=updated')
        self.assertEqual(
            update_query_string('/foo/?bar=baz', {'bar': 'updated'}),
            '/foo/?bar=updated')
        self.assertEqual(
            update_query_string('/foo/', {'bar': 'created'}),
            '/foo/?bar=created')

    def test_tag_update_query_string(self):
        template = Template(
            "{% load updateurl %}{% update_query_string with page=num_page %}"
        )
        self.assertEqual(
            template.render(
                Context(
                    {'request': RequestFactory().get("/fake"),
                     'page': 'page', 'num_page': 2}
                )
            ),
            u'/fake?page=2')

    def test_is_checkbox(self):
        class MockForm(forms.Form):
            a = forms.CharField()
            b = forms.BooleanField()

        form = MockForm()

        boundfield_a, boundfield_b = form.visible_fields()

        self.assertEqual("a", boundfield_a.name)
        self.assertFalse(is_checkbox(boundfield_a))

        self.assertEqual("b", boundfield_b.name)
        self.assertTrue(is_checkbox(boundfield_b))



import unittest

from django import forms
from django_genericfilters import forms as gf


class FieldTestCase(unittest.TestCase):

    def setUp(self):

        class Form(forms.Form):
            test = gf.ChoiceField(label='Test')

        self.form = Form()

    def test_filtered_choice_field_required(self):
        self.assertFalse(self.form.fields['test'].required)

    def test_filtered_choice_field_auto_choices(self):
        choices = self.form.fields['test'].choices

        self.assertEqual(choices[0][0], 'yes')
        self.assertEqual(choices[0][1], 'Test')

        self.assertEqual(choices[1][0], 'no')
        self.assertEqual(choices[1][1], 'No Test')



"""Template tag library.

Provides one template tag: "check_class".

Example:

.. code-block:: Django

    {% load utils %}

    {% if form.field|is_checkbox %}

"""

from django import template

register = template.Library()


@register.filter
def is_checkbox(form_field):
    return form_field.field.widget.__class__.__name__ == 'CheckboxInput'



import unittest

from django import forms

from django_genericfilters import forms as gf


class FormTestCase(unittest.TestCase):

    def test_query_form_mixin(self):
        class Form(gf.QueryFormMixin, forms.Form):
            pass

        form = Form()

        self.assertTrue('query' in form.fields)

    def test_order_form_mixin(self):
        def get_order_by_choices():
            return (('last_name', 'Last Name'),
                    ('first_name', 'First Name'))

        class Form(gf.OrderFormMixin, forms.Form):
            def get_order_by_choices(self):
                return get_order_by_choices()

        form = Form()

        self.assertTrue('order_by' in form.fields)
        self.assertTrue('order_reverse' in form.fields)

        self.assertEqual(form.fields['order_by'].choices,
                         list(get_order_by_choices()))

    def test_get_order_by_choices_not_implemented(self):
        class Form(gf.OrderFormMixin):
            pass

        self.assertRaises(NotImplementedError, Form)


class FilteredFormTestCase(unittest.TestCase):

    # Define a form class for this test case
    class Form(gf.FilteredForm):

        def get_order_by_choices(self):
            return (('last_name', 'Last Name'),
                    ('first_name', 'First Name'))

    def test_empty_form_bound(self):
        form = self.Form(data={}, initial={"order_by": 'last_name'})
        self.assertTrue(form.is_bound)
        self.assertTrue(form.is_valid())
        self.assertEqual(form.cleaned_data['order_by'], 'last_name')

    def test_form_order_by_bound(self):
        form = self.Form(data={'order_by': 'first_name', 'order_reverse': 0},
                         initial={"order_by": 'last_name'})
        self.assertTrue(form.is_bound)
        self.assertTrue(form.is_valid())
        self.assertEqual(form.cleaned_data['order_by'], 'first_name')

    def test_filtered_form(self):
        form = self.Form()

        self.assertEqual(str(type(form.fields['query'].widget)),
                         "<class 'django.forms.widgets.HiddenInput'>")

        self.assertEqual(str(type(form.fields['order_by'].widget)),
                         "<class 'django.forms.widgets.HiddenInput'>")

        self.assertEqual(str(type(form.fields['order_reverse'].widget)),
                         "<class 'django.forms.widgets.HiddenInput'>")



from django import forms
from django.utils.translation import ugettext_lazy as _


class ChoiceField(forms.ChoiceField):

    def __init__(self, *args, **kwargs):

        kwargs.setdefault('required', False)

        if 'choices' not in kwargs:
            kwargs['choices'] = (
                ('yes', kwargs.get('label', _('Yes'))),
                ('no', _('No %(label)s') % {'label': kwargs.get('label', '')}))
        super(ChoiceField, self).__init__(*args, **kwargs)



"""
django generic filters implement a set of mixin to work with ordered
and filtered queryset.

"""
from django import forms
from django.utils.translation import ugettext_lazy as _

from .fields import *  # NOQA


class QueryFormMixin(object):
    """
    Mixin implementing a query parameters for filtering results.
    """
    def __init__(self, *args, **kwargs):
        super(QueryFormMixin, self).__init__(*args, **kwargs)
        self.fields['query'] = forms.CharField(required=False,
                                               widget=forms.HiddenInput)


class OrderFormMixin(object):
    """
    Mixin implementing order_by and order_by_reverse for your filtered
    results
    """

    def __init__(self, *args, **kwargs):
        super(OrderFormMixin, self).__init__(*args, **kwargs)

        self.fields['order_by'] = forms.ChoiceField(
            label=_('order by'),
            required=False,
            widget=forms.HiddenInput,
            choices=self.get_order_by_choices())
        self.fields['order_reverse'] = forms.BooleanField(
            label=_('order by'),
            required=False,
            widget=forms.HiddenInput)

    def clean_order_by(self):
        if self['order_by'].html_name not in self.data:
            return self.initial.get(self['order_by'].html_name,
                                    self.fields['order_by'].initial)
        return self.cleaned_data['order_by']

    def get_order_by_choices(self):
        """
        If you use OrderFormMixin, this method must be implemented in
        your form.

        get_order_by choices should return a list of tuples. Those
        will be used as choices for the order_by field.

        Example:

        .. code-block:: python

            def get_order_by_choices(self):
                return [("1", "choice1"),
                        ("2", "choice2")]
        """

        raise NotImplementedError(
            _("Don't forget to implements get_order_by_choices"))


def clean_yesno(value):
    if isinstance(value, str):
        return {"yes": True, "no": False}.get(value, value)

    return value


class FilteredForm(OrderFormMixin, QueryFormMixin, forms.Form):
    """
    FilteredForm is like a classic forms. But It use OrderFormMixin
    and QueryFormMixin
    """

    def clean(self):
        data = self.cleaned_data
        update = {key: clean_yesno(value) for key, value in data.items()}
        data.update(update)
        return data



import urllib

from django import forms
from django.db import models
from django.http import QueryDict
from django.test import RequestFactory, TestCase
from django.utils.datastructures import MultiValueDict

from django_genericfilters import views
from django_genericfilters.forms import FilteredForm


def setup_view(view, request, *args, **kwargs):
    """Mimic as_view() returned callable, but returns view instance.

    args and kwargs are the same you would pass to ``reverse()``

    See also: https://code.djangoproject.com/ticket/20456

    """

    view.request = request
    view.args = args
    view.kwargs = kwargs

    return view


class ParentModel(models.Model):
    """
    define a parent model
    """
    name = models.CharField(max_length=250)


class StatusModel(models.Model):
    """
    define a dummy status model
    """
    name = models.CharField(max_length=250)


class FilteredViewTestCase(TestCase):

    class QueryModel(models.Model):
        """
        Define a dummy model for this test case
        """
        people = models.ForeignKey(ParentModel, on_delete=models.CASCADE)
        city = models.CharField(max_length=250)
        country = models.CharField(max_length=250)
        organization = models.CharField(max_length=250)
        status = models.ForeignKey(
            StatusModel, null=True, on_delete=models.CASCADE
        )

    class Form(FilteredForm):

        city = forms.ChoiceField(
            label='city', required=False,
            choices=(
                ("N", "Nantes"),
                ("P", "Paris")
            )
        )

        country = forms.ChoiceField(
            label='country', required=False,
            choices=(
                ("F", "France"),
                ("P", "Portugal")
            )
        )

        people = forms.ChoiceField(
            label='people', required=False,
            choices=(
                ("S", "Some"),
                ("A", "Any")
            )
        )

        organization = forms.MultipleChoiceField(
            label='organization', required=False,
            choices=(
                ('A', 'A Team'),
                ('B', 'B Team'),
                ('C', 'C Team')
            ))

        parent = forms.ModelChoiceField(
            queryset=ParentModel.objects.all(),
            label='parent', required=False
        )

        status = forms.ModelMultipleChoiceField(
            label='status',
            required=False,
            queryset=StatusModel.objects.all()
        )

        def get_order_by_choices(self):
            return (('last_name', 'Last Name'),
                    ('first_name', 'First Name'))

    def test_default_order_fallback_form_valid(self):
        """Queryset is unordered if no default_order or data (valid form)."""
        data = {"city": "N"}
        view = setup_view(
            views.FilteredListView(
                model=self.QueryModel, form_class=self.Form),
            RequestFactory().get('/fake', data))

        view.form.is_valid()
        queryset = view.form_valid(view.form)
        self.assertEqual(list(queryset.query.order_by), [])

    def test_default_order_fallback_form_invalid(self):
        """Queryset is unordered if no default_order or data (invalid form)."""
        data = {"city": "fake"}
        view = setup_view(
            views.FilteredListView(
                model=self.QueryModel, form_class=self.Form),
            RequestFactory().get('/fake', data))

        view.form.is_valid()
        queryset = view.form_invalid(view.form)
        self.assertEqual(list(queryset.query.order_by), [])

    def test_default_order_fallback_form_empty(self):
        """Queryset is unordered if no default_order or data (empty form)."""
        request = RequestFactory().get('/fake')
        view = setup_view(
            views.FilteredListView(
                model=self.QueryModel, form_class=self.Form),
            request)

        queryset = view.form_empty()
        self.assertEqual(list(queryset.query.order_by), [])

    def test_default_filter(self):
        """Test the default filter"""
        request = RequestFactory().get('/fake')
        view = setup_view(
            views.FilteredListView(
                model=self.QueryModel, form_class=self.Form,
                default_filter={'is_active': '1', 'page': '1'}),
            request)

        query_filter = urllib.parse.urlencode({'is_active': '1', 'page': '1'})
        get_filter = view.get_form_kwargs()
        self.assertEqual(get_filter['data'], QueryDict(query_filter))

        def test_default_filter_submit(self):
            """Test the default filter submit"""
            data = {"city": "N"}
            request = RequestFactory().get('/fake', data)
            view = setup_view(
                views.FilteredListView(
                    model=self.QueryModel, form_class=self.Form,
                    default_filter={'is_active': '1', 'page': '1'}),
                request)

            query_filter = urllib.parse.urlencode({
                'is_active': '1', 'page': '1', 'city': 'N'})
            get_filter = view.get_form_kwargs()
            self.assertEqual(get_filter['data'], QueryDict(query_filter))

    def test_default_order_form_valid(self):
        """Queryset is ordered by default_order when no order_by in request."""
        data = {"city": "N"}
        view = setup_view(
            views.FilteredListView(
                model=self.QueryModel,
                form_class=self.Form,
                default_order='last_name'),
            RequestFactory().get('/fake', data))

        view.form.is_valid()
        queryset = view.form_valid(view.form)
        self.assertEqual(list(queryset.query.order_by), ['last_name'])

    def test_default_order_form_invalid(self):
        """Queryset is ordered by default_order when no order_by in request
        and form is invalid."""
        data = {"city": "fake"}
        view = setup_view(
            views.FilteredListView(
                model=self.QueryModel,
                form_class=self.Form,
                default_order='last_name'),
            RequestFactory().get('/fake', data))

        view.form.is_valid()
        queryset = view.form_invalid(view.form)
        self.assertEqual(list(queryset.query.order_by), ['last_name'])

    def test_default_order_form_empty(self):
        """Queryset is ordered by default_order when no order_by in request."""
        request = RequestFactory().get('/fake')
        view = setup_view(
            views.FilteredListView(
                model=self.QueryModel,
                form_class=self.Form,
                default_order='last_name'),
            request)

        queryset = view.form_empty()
        self.assertEqual(list(queryset.query.order_by), ['last_name'])

    def test_default_order_reverse(self):
        """To test order reverse"""
        data = {"city": "N"}
        view = setup_view(
            views.FilteredListView(
                model=self.QueryModel,
                form_class=self.Form,
                default_order='-last_name'),
            RequestFactory().get('/fake', data))

        view.form.is_valid()
        queryset = view.form_valid(view.form)
        self.assertEqual(list(queryset.query.order_by), ['-last_name'])

    def test_default_order_in_request(self):
        """Test with order_by in data."""
        data = {"city": "N", "order_by": "last_name"}
        view = setup_view(
            views.FilteredListView(
                model=self.QueryModel,
                form_class=self.Form,
                default_order='-last_name'),
            RequestFactory().get('/fake', data))

        view.form.is_valid()
        queryset = view.form_valid(view.form)
        self.assertEqual(list(queryset.query.order_by), ['last_name'])

    def test_filtered_list_view(self):
        a = views.FilteredListView(filter_fields=['city'],
                                   form_class=self.Form,
                                   model=self.QueryModel)

        b = views.FilteredListView(filter_fields=['city', 'people'],
                                   qs_filter_fields={'people__name': 'people'},
                                   form_class=self.Form,
                                   model=self.QueryModel)
        setattr(
            a,
            'request',
            type('obj', (object, ), {"method": "GET", "GET": {"city": "N"}})
        )

        setattr(
            b,
            'request',
            type('obj', (object, ), {"method": "GET", "GET": {"people": "S"}})
        )

        self.assertEqual({'city': 'city'}, a.get_qs_filters())
        a.form.is_valid()
        self.assertIn(
            'WHERE "django_genericfilters_querymodel"."city" = N',
            a.form_valid(a.form).query.__str__()
        )

        self.assertEqual({'people__name': 'people'}, b.get_qs_filters())
        b.form.is_valid()
        self.assertIn(
            'WHERE "django_genericfilters_parentmodel"."name" = S',
            b.form_valid(b.form).query.__str__()
        )

    def test_filtered_list_view__none(self):
        """
            FIXED : None value add "IS NULL" filters instead of ignore it.
        """
        view = views.FilteredListView(qs_filter_fields={
                                          'city': 'city',
                                          'people__name': 'people'
                                      },
                                      form_class=self.Form,
                                      model=self.QueryModel)

        data = {"city": "None", "people": "S"}
        setup_view(view, RequestFactory().get('/fake', data))
        view.form.is_valid()
        self.assertIn(
            'WHERE "django_genericfilters_parentmodel"."name" = S',
            str(view.form_valid(view.form).query)
        )

        view = views.FilteredListView(qs_filter_fields={
                                          'city': 'city',
                                          'people__name': 'people'
                                      },
                                      form_class=self.Form,
                                      model=self.QueryModel)

        data = {"city": "N", "people": "None"}
        setup_view(view, RequestFactory().get('/fake', data))
        view.form.is_valid()
        self.assertIn(
            'WHERE "django_genericfilters_querymodel"."city" = N',
            str(view.form_valid(view.form).query)
        )

    def test_filtered_list_view__multiplechoice(self):
        """
            FIXED : filtered fields has HiddenWidget widgets that cannot handle
                    multiple values. Use Field.hidden_widget instead.
        """
        view = views.FilteredListView(filter_fields=['organization'],
                                      form_class=self.Form,
                                      model=self.QueryModel)

        data = MultiValueDict({"organization": ['A']})
        setup_view(view, RequestFactory().get('/fake', data))

        self.assertTrue(view.form.is_valid(), view.form.errors)
        self.assertIn(
            'WHERE "django_genericfilters_querymodel"."organization" IN (A)',
            str(view.form_valid(view.form).query)
        )

        view = views.FilteredListView(filter_fields=['organization'],
                                      form_class=self.Form,
                                      model=self.QueryModel)

        data = MultiValueDict({"organization": ['A', 'C']})
        setup_view(view, RequestFactory().get('/fake', data))
        self.assertTrue(view.form.is_valid())

    def test_filtered_list_view__multiplechoice__qs_filter_field(self):
        """
            FIXED : When using qs_filter_field, the behaviour changes because
                    the HiddenWidget trick only works with filter_field
                    attribute. But it compares a list with EQUAL operator
                    instead of IN.
        """
        people = ParentModel.objects.create(name='fake')

        self.QueryModel.objects.create(organization='A', people=people)
        self.QueryModel.objects.create(organization='C', people=people)

        view = views.FilteredListView(qs_filter_fields={
                                          'organization': 'organization'
                                      },
                                      form_class=self.Form,
                                      model=self.QueryModel)

        data = MultiValueDict({"organization": ['A']})
        setup_view(view, RequestFactory().get('/fake', data))

        self.assertTrue(view.form.is_valid(), view.form.errors)
        self.assertIn(
            'WHERE "django_genericfilters_querymodel"."organization" IN (A)',
            str(view.form_valid(view.form).query)
        )
        self.assertEqual(1, view.form_valid(view.form).count())

        view = views.FilteredListView(qs_filter_fields={
                                          'organization': 'organization'
                                      },
                                      form_class=self.Form,
                                      model=self.QueryModel)

        data = MultiValueDict({"organization": ['A', 'C']})
        setup_view(view, RequestFactory().get('/fake', data))
        self.assertTrue(view.form.is_valid())
        self.assertEqual(2, view.form_valid(view.form).count())

    def test_filtered_list_view__modelchoice(self):
        peopleA = ParentModel.objects.create(name='fakeA')

        view = views.FilteredListView(qs_filter_fields={
                                          'city': 'city',
                                          'people': 'parent'
                                      },
                                      form_class=self.Form,
                                      model=self.QueryModel)

        data = {"parent": peopleA.pk}
        setup_view(view, RequestFactory().get('/fake', data))
        view.form.is_valid()
        self.assertIn(
            'WHERE "django_genericfilters_querymodel"."people_id" = %s' % (
                peopleA.pk,
            ),
            str(view.form_valid(view.form).query)
        )

    def test_filtered_list_view__modelchoice__empty_queryset(self):
        """
            FIXED : Empty queryset in ModelChoiceField add "IS NULL" filters
                    instead of ignore it.
        """
        view = views.FilteredListView(qs_filter_fields={
                                          'city': 'city',
                                          'people': 'parent'
                                      },
                                      form_class=self.Form,
                                      model=self.QueryModel)

        data = {"city": "N", "parent": 1}
        setup_view(view, RequestFactory().get('/fake', data))
        view.form.is_valid()
        self.assertIn(
            'WHERE "django_genericfilters_querymodel"."city" = N',
            str(view.form_valid(view.form).query)
        )

    def test_filtered_list_view__modelchoice__none(self):
        """
            FIXED : Empty queryset in ModelChoiceField add "IS NULL" filters
                    instead of ignore it.
        """
        view = views.FilteredListView(qs_filter_fields={
                                          'city': 'city',
                                          'people': 'parent'
                                      },
                                      form_class=self.Form,
                                      model=self.QueryModel)

        data = {"city": "N", "parent": "None"}
        setup_view(view, RequestFactory().get('/fake', data))
        view.form.is_valid()
        self.assertIn(
            'WHERE "django_genericfilters_querymodel"."city" = N',
            str(view.form_valid(view.form).query)
        )

    def test_filtered_list_view__multiplemodelchoice(self):
        """
            FIXED : filtered fields has HiddenWidget widgets that cannot handle
                    multiple values. Use Field.hidden_widget instead.
        """
        stateA = StatusModel.objects.create(name='stateA')
        stateB = StatusModel.objects.create(name='stateB')
        stateC = StatusModel.objects.create(name='stateC')
        people = ParentModel.objects.create(name='fake')

        A = self.QueryModel.objects.create(organization='A', people=people,
                                           status=stateA)
        B = self.QueryModel.objects.create(organization='B', people=people,
                                           status=stateB)
        C = self.QueryModel.objects.create(organization='C', people=people,
                                           status=stateB)
        self.QueryModel.objects.create(organization='D', people=people,
                                       status=stateC)

        view = views.FilteredListView(filter_fields=["city", 'status'],
                                      form_class=self.Form,
                                      model=self.QueryModel)

        data = MultiValueDict({"status": [stateA.pk]})
        setup_view(view, RequestFactory().get('/fake', data))
        view.form.is_valid()

        self.assertIsInstance(view.form.fields['status'].widget,
                              forms.MultipleHiddenInput)

        queryset = view.form_valid(view.form)
        self.assertIn('IN (%s)' % stateA.pk, str(queryset.query))
        self.assertEqual([A], list(queryset.all()))

        view = views.FilteredListView(filter_fields=["city", 'status'],
                                      form_class=self.Form,
                                      model=self.QueryModel)

        data = MultiValueDict({"status": [stateA.pk, stateB.pk]})
        setup_view(view, RequestFactory().get('/fake', data))
        view.form.is_valid()

        queryset = view.form_valid(view.form)
        self.assertEqual([A, B, C], list(queryset.all()))

    def test_filtered_list_view__multiplemodelchoice__qs_filter_field(self):
        stateA = StatusModel.objects.create(name='stateA')
        stateB = StatusModel.objects.create(name='stateB')
        stateC = StatusModel.objects.create(name='stateC')
        people = ParentModel.objects.create(name='fake')

        A = self.QueryModel.objects.create(organization='A', people=people,
                                           status=stateA)
        B = self.QueryModel.objects.create(organization='B', people=people,
                                           status=stateB)
        C = self.QueryModel.objects.create(organization='C', people=people,
                                           status=stateB)
        self.QueryModel.objects.create(organization='D', people=people,
                                       status=stateC)

        view = views.FilteredListView(qs_filter_fields={
                                          "city": "city",
                                          "status": "status"
                                      },
                                      form_class=self.Form,
                                      model=self.QueryModel)

        data = MultiValueDict({"status": [stateA.pk]})
        setup_view(view, RequestFactory().get('/fake', data))
        view.form.is_valid()

        self.assertIsInstance(view.form.fields['status'].widget,
                              forms.SelectMultiple)

        queryset = view.form_valid(view.form)
        self.assertIn('IN (%s)' % stateA.pk, str(queryset.query))
        self.assertEqual([A], list(queryset))

        view = views.FilteredListView(qs_filter_fields={
                                          "city": "city",
                                          "status": "status"
                                      },
                                      form_class=self.Form,
                                      model=self.QueryModel)

        data = MultiValueDict({"status": [stateA.pk, stateB.pk]})
        setup_view(view, RequestFactory().get('/fake', data))
        view.form.is_valid()

        queryset = view.form_valid(view.form)
        self.assertEqual([A, B, C], list(queryset))

    def test_filtered_list_view__multiplemodelchoice__invalid_id(self):
        """
            FIXED : Invalid id in MultipleModelChoiceField generate a None
                    value and add "IS NULL" filter instead of ignore it.
        """
        StatusModel.objects.create(name='stateA')
        StatusModel.objects.create(name='stateB')

        view = views.FilteredListView(qs_filter_fields={
                                          "city": "city",
                                          'status': 'status'
                                      },
                                      form_class=self.Form,
                                      model=self.QueryModel)

        data = MultiValueDict({"status": [1001]})
        setup_view(view, RequestFactory().get('/fake', data))
        view.form.is_valid()
        # no filter at all
        self.assertNotIn('WHERE', str(view.form_valid(view.form).query))

    def test_filtered_list_view__multiplemodelchoice__none(self):
        """
            FIXED : Empty queryset in MultipleModelChoiceField is added as
                    subrequest in filter and raises an sql error instead of
                    ignore it.
        """
        people = ParentModel.objects.create(name='fake')

        statusA = StatusModel.objects.create(name='stateA')
        statusB = StatusModel.objects.create(name='stateB')

        self.QueryModel.objects.create(organization='A', people=people,
                                       status=statusA)
        self.QueryModel.objects.create(organization='C', people=people,
                                       status=statusB)

        view = views.FilteredListView(qs_filter_fields={
                                          "city": "city",
                                          'status': 'status'
                                      },
                                      form_class=self.Form,
                                      model=self.QueryModel)

        setup_view(view, RequestFactory().get('/fake', {}))
        view.form.is_valid()
        # no filter at all
        self.assertNotIn('WHERE', str(view.form_valid(view.form).query))

    def test_is_form_submitted_method(self):
        """Is form submitted return True when the request method is GET."""
        request = RequestFactory().get('/fake', {"foo": "bar"})
        view = setup_view(views.FilteredListView(), request)
        assert view.is_form_submitted() is True

        request = RequestFactory().post('/fake', {"foo": "bar"})
        view = setup_view(views.FilteredListView(), request)
        assert view.is_form_submitted() is False

    def test_is_form_submitted_no_args(self):
        """Is form submitted return False when the queryset is empty."""
        request = RequestFactory().get('/fake')
        view = setup_view(views.FilteredListView(), request)
        assert view.is_form_submitted() is False



#  Based on: http://www.djangosnippets.org/snippets/73/
#
#  Modified by Sean Reifschneider to be smarter about surrounding page
#  link context.  For usage documentation see:
#
#     http://www.tummy.com/Community/Articles/django-pagination/

from django import template

register = template.Library()


def paginator(context, adjacent_pages=1):
    """
    To be used in conjunction with the object_list generic view.

    Adds pagination context variables for use in displaying first, adjacent and
    last page links in addition to those created by the object_list generic
    view.

    """
    if 'page_obj' in context:
        page_obj = context['page_obj']
        paginator = context['paginator']
        startPage = max(page_obj.number - adjacent_pages, 1)
        if startPage <= 3:
            startPage = 1
        endPage = page_obj.number + adjacent_pages + 1
        if endPage >= paginator.num_pages - 1:
            endPage = paginator.num_pages + 1
        page_numbers = [n for n in range(startPage, endPage)
                        if n > 0 and n <= paginator.num_pages]

        context['page_numbers'] = page_numbers
        context['show_first'] = 1 not in page_numbers
        context['show_last'] = paginator.num_pages not in page_numbers
        return context
    else:
        return {}


register.inclusion_tag('snippets/pagination.html',
                       takes_context=True)(paginator)



import os
from setuptools import setup


def read_relative_file(filename):
    """Returns contents of the given file, which path is supposed relative
    to this module."""
    with open(os.path.join(os.path.dirname(__file__), filename)) as f:
        return f.read().strip()


NAME = 'django-generic-filters'
README = read_relative_file('README.rst')
VERSION = read_relative_file('VERSION')
PACKAGES = ['django_genericfilters']
REQUIRES = [
    'Django',
    'munch',
]


if __name__ == '__main__':  # Don't run setup() when we import this module.
    setup(name=NAME,
          version=VERSION,
          description='Easy filters for your Generic ListView with Django.',
          long_description=README,
          classifiers=['Development Status :: 4 - Beta',
                       'License :: OSI Approved :: BSD License',
                       'Programming Language :: Python :: 3',
                       'Programming Language :: Python :: 3.6',
                       'Programming Language :: Python :: 3.7',
                       'Programming Language :: Python :: 3.8',
                       'Framework :: Django',
                       ],
          keywords='class-based view, generic view, filters',
          author='Novapost Team',
          author_email='peopleask@novapost.fr',
          url='https://github.com/novapost/%s' % NAME,
          license='BSD',
          packages=PACKAGES,
          include_package_data=True,
          install_requires=REQUIRES,
          setup_requires=['setuptools'],
          )



from django import forms
from django.http import QueryDict
from django.db.models import Q, QuerySet
from django.views.generic import ListView
from django.views.generic.edit import FormMixin
from django.utils.translation import ugettext_lazy as _

from munch import Munch


EMPTY_FILTER_VALUES = (None, '', '-1')


def is_filter(value, form):
    return bool(value in form.cleaned_data and form.cleaned_data[value])


class FilteredListView(FormMixin, ListView):
    """A Generic ListView used to filter and order objects."""
    default_order = None
    default_filter = None

    def is_form_submitted(self):
        """
        Return True if the form is already submited. False otherwise
        """
        return bool(self.request.method == 'GET' and self.request.GET)

    def get_initial(self):
        """
        add "order_by" and "order_reverse" to the initials.
        """
        kwargs = super(FilteredListView, self).get_initial()

        if self.default_order:
            order_by = self.default_order

            kwargs['order_by'] = order_by
            kwargs['order_reverse'] = order_by.startswith('-')

        return kwargs

    def get_form_kwargs(self):
        """Read GET data to return keyword arguments for the form."""
        kwargs = {'initial': self.get_initial()}
        data = QueryDict({}).copy()

        if self.default_filter:
            data.update(self.default_filter)

        if self.is_form_submitted():
            data.update(self.request.GET)

        kwargs.update({'data': data})

        return kwargs

    def __get_queryset(self):
        """Helper to get ListView default queryset."""
        return super(ListView, self).get_queryset()

    def get_queryset(self):
        """Return filtered queryset. Uses form_valid() or form_invalid()."""
        if self.form.is_valid():
            return self.form_valid(self.form)
        else:
            return self.form_invalid(self.form)

    def get_qs_filters(self):
        """
        retreive filters from "qs_filter_fields" or "filter_fields"
        and return them as a dict to be used by self.form_valid
        """

        filters = {}

        if hasattr(self, 'qs_filter_fields'):
            filters = self.qs_filter_fields

        elif hasattr(self, 'filter_fields'):
            for field in self.filter_fields:
                filters[field] = field

        return filters

    def clean_qs_filter_field(self, key, value):
        if value in EMPTY_FILTER_VALUES:
            return None

        if isinstance(value, QuerySet):
            if value.exists():
                return {'%s__in' % key: value}
        elif isinstance(value, (tuple, list)):
            if len(value) > 0:
                return {'%s__in' % key: value}
        else:
            return {key: value}

    def form_valid(self, form):
        """
        The form_valid is reponsible for filtering and ordering the
        base queryset. It return a queryset.

        :param: `django.forms.Forms`
        :return: `django.db.models.query.QuerySet`
        """
        # Get default queryset from ListView parameters (queryset, model, ...)
        queryset = self.__get_queryset()

        # Handle QueryFormMixin
        if is_filter('query', form):
            query = form.cleaned_data['query']
            query_words = query.split()
            filters = None
            for f in self.search_fields:
                for word in query_words:
                    q = Q(**{f + '__icontains': word})
                    filters = filters | q if filters else q
            if filters:
                queryset = queryset.filter(filters)

        # Handle get_qs_filters
        filters = {}
        extra_conditions = getattr(self, 'qs_filter_fields_conditions', None)
        clean_qs_filter_field = self.clean_qs_filter_field

        for k, v in self.get_qs_filters().items():
            qs_filter = clean_qs_filter_field(k, form.cleaned_data.get(v))
            if qs_filter is not None:
                filters.update(qs_filter)

                # Get extra condition for a field to on the filters
                if extra_conditions is not None:
                    filter_fields_conditions = extra_conditions.get(k, {})

                    for key, value in filter_fields_conditions.items():
                        filters[key] = value

        queryset = queryset.filter(**filters)

        # Handle OrderFormMixin
        if is_filter('order_by', form):
            order_field = form.cleaned_data['order_by']
            queryset = queryset.order_by(order_field)

        if is_filter('order_reverse', form):
            queryset = queryset.reverse()

        return queryset

    def form_invalid(self, form):
        """Return queryset when submitted form is invalid.

        Default implementation uses :meth:`form_empty`.

        :param: `django.forms.Forms`
        :return: `django.db.models.query.QuerySet`

        """
        return self.form_empty()

    def form_empty(self):
        """Return queryset used when form is not submitted."""
        queryset = self.__get_queryset()
        if self.default_order:
            queryset = queryset.order_by(self.default_order)

        return queryset

    @property
    def form(self):
        """
        guess the form to be used and hide the filter_fields in the template

        """
        try:
            return self._form
        except AttributeError:
            form_class = self.get_form_class()
            self._form = self.get_form(form_class)

            # Hide filter_fields
            if hasattr(self, 'filter_fields'):
                for fieldname in self.filter_fields:
                    field = self._form.fields[fieldname]
                    hidden_widget = getattr(field, 'hidden_widget',
                                            forms.HiddenInput)
                    field.widget = hidden_widget()

            return self._form

    def get_context_data(self, **kwargs):
        """
        Add a list of filters and self.form to the context to be rendered by
        the view.
        """
        kwargs = ListView.get_context_data(self, **kwargs)
        kwargs['form'] = self.form
        kwargs['filters'] = self.get_filters()
        kwargs['stacked_fields'] = getattr(self, 'stacked_fields', [])

        return kwargs

    def get_filters(self):
        """
        Convert some ChoiceField in a list of choices in the
        template.
        """
        filters = []

        if hasattr(self, 'filter_fields'):
            for field in self.filter_fields:
                new_filter = Munch()
                new_filter.label = self.form.fields[field].label
                new_filter.name = field
                new_filter.choices = []
                selected = False
                for choice in self.form.fields[field].choices:
                    new_choice = Munch()
                    new_choice.value = choice[0]
                    new_choice.label = choice[1]
                    yesno = {"yes": True, "no": False}
                    if hasattr(self.form, 'cleaned_data') and \
                            field in self.form.cleaned_data:
                        # Get value for ModelChoiceField or ChoiceField
                        value = getattr(self.form.cleaned_data[field], 'pk',
                                        self.form.cleaned_data[field])
                        if value == yesno.get(choice[0], choice[0]):
                            new_choice.is_selected = True
                            selected = True
                        else:
                            new_choice.is_selected = False
                    else:
                        new_choice.is_selected = False
                    new_filter.choices.append(new_choice)

                if not self.form.fields[field].required and \
                        not [c for c in new_filter.choices
                             if c.value == '-1' or c.value == '']:
                    all_choice = Munch(value='', label=_('All'),
                                       is_selected=(not selected))
                    new_filter.choices.insert(0, all_choice)
                filters.append(new_filter)

        return filters
